import java.util.Objects;
import java.lang.Exception;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;

import java.net.URL;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;

import java.util.ArrayList;

public class Main {
    public static void main(String[] args) {
        boolean help_enqueued = false;
        boolean allow_networking = true;
        String matrix_file_name = "";
        int verbosity = 1;
        
        int p = 0;
        while (p < args.length) {
            try {
                if (Objects.equals(args[p], "-h")) {    
                    help_enqueued = true;
                    p += 1;
                }
                else if (Objects.equals(args[p], "-v")) {
                    verbosity = Integer.parseInt(args[p+1]);
                    p += 2;
                }
                else if (Objects.equals(args[p], "--offline")) {
                    allow_networking = false;
                    p += 2;
                }
                else {
                    matrix_file_name = args[p++];
                }
            }
            catch (Exception e) {
                System.err.println("[ERROR] Exception while parsing CLI arguments: " + e);
                System.err.println("[ERROR] Aborting further execution.");
                System.exit(1);
            }
        }

        if (help_enqueued) {
            printHelp(allow_networking);
            System.exit(0);
        }

        if (Objects.equals("", matrix_file_name)) {
            System.err.println("[WARN] No filename specified to read from, using default one");
            matrix_file_name = "random_matrix.txt";
        }

        File matrix_file = new File(matrix_file_name);
        if (!matrix_file.exists()) {
            if (allow_networking) {
                System.err.println("[WARN] File is missing, downloading random matrix from the server");
                fetchResource("http://lab2.kpi.dev:16554/matrix.py", matrix_file_name);
            }
        }

    }

    private static void fetchResource(String remote_url, String output_filename) {
        try {
            BufferedInputStream in = new BufferedInputStream(new URL(remote_url).openStream());
            FileOutputStream fileOutputStream = new FileOutputStream(output_filename);
    
            byte dataBuffer[] = new byte[1024];
            int bytesRead;
            while ((bytesRead = in.read(dataBuffer, 0, 1024)) != -1) {
                fileOutputStream.write(dataBuffer, 0, bytesRead);
            }
        } catch (Exception e) {
            System.out.println("[ERROR] Failed to fetch resource " + output_filename + "from " + remote_url + " due to the following exception: " + e);
        }
    }

    private static void printHelp(boolean allow_net) {
        try {
            File help_file = new File("src/help.txt");

            if (!help_file.exists()) {
                System.err.println("[WARN] Help file is missing.");
                if (allow_net) {
                    System.err.println("[INFO] Trying to recover it from the git server");
                    fetchResource("http://lab2.kpi.dev:3000/dymik739/oop-labs-collection/raw/branch/lab2-dev/labs/2/src/help.txt", "src/help.txt");
                } else {
                    System.err.println("[INFO] Networking is disabled, not recovering");
                    System.exit(1);
                }
            }
    
            Scanner help_file_scanner = new Scanner(help_file);

            while (help_file_scanner.hasNextLine()) {
                System.out.print(help_file_scanner.nextLine());
            }
    
            help_file_scanner.close(); 
        } catch (Exception e) {
            System.out.println("[ERROR] Failed to read help due to the following exception: " + e);
            System.exit(1);
        }
    }
}
